'''
PyQt를 사용한 실시간 그래프 구현부터 설명드리겠습니다. PyQt는 더 반응성이 좋고 사용자 인터페이스를 커스터마이징하기 쉽습니다.
'''

import sys
import socket
import json
import numpy as np

# from PyQt5.QtWidgets import QApplication, QMainWindow, QVBoxLayout, QWidget
# from PyQt5.QtCore import QTimer

from PyQt6.QtWidgets import QApplication, QMainWindow, QVBoxLayout, QWidget
from PyQt6.QtCore import QTimer

from pyqtgraph import PlotWidget, plot

class FFTVisualizerQt(QMainWindow):
    def __init__(self, port=5000):
        super().__init__()
        self.setWindowTitle('FFT Spectrum Analyzer')
        self.setGeometry(100, 100, 800, 600)

        # 중앙 위젯 설정
        central_widget = QWidget()
        self.setCentralWidget(central_widget)
        layout = QVBoxLayout(central_widget)

        # pyqtgraph 플롯 위젯 설정
        self.plot_widget = PlotWidget()
        layout.addWidget(self.plot_widget)
        
        # 플롯 설정
        self.plot_widget.setBackground('white')
        self.plot_widget.showGrid(x=True, y=True)
        self.plot_widget.setLabel('left', 'Amplitude')
        self.plot_widget.setLabel('bottom', 'Frequency (Hz)')
        self.plot_widget.setTitle('Real-time FFT Spectrum', color='black', size='12pt')
        
        # 플롯 곡선
        self.curve = self.plot_widget.plot(pen='b')

        # Socket 설정
        self.server_socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.server_socket.bind(('localhost', port))
        self.server_socket.listen(1)
        print(f"Waiting for connection on port {port}...")
        
        # 소켓을 non-blocking 모드로 설정
        self.server_socket.setblocking(False)
        
        # 클라이언트 연결을 위한 타이머
        self.connect_timer = QTimer()
        self.connect_timer.timeout.connect(self.try_connect)
        self.connect_timer.start(100)  # 100ms 간격으로 연결 시도
        
        self.client_socket = None
        self.connected = False

        # 데이터 업데이트 타이머
        self.update_timer = QTimer()
        self.update_timer.timeout.connect(self.update_plot)
        self.update_timer.start(50)  # 50ms 간격으로 업데이트 (20 FPS)

    def try_connect(self):
        if not self.connected:
            try:
                self.client_socket, _ = self.server_socket.accept()
                self.client_socket.setblocking(False)
                self.connected = True
                print("Client connected!")
                self.connect_timer.stop()
            except BlockingIOError:
                pass  # 연결 대기 중
            except Exception as e:
                print(f"Connection error: {e}")

    def update_plot(self):
        if not self.connected:
            return

        try:
            # 데이터 수신 시도
            # data = self.client_socket.recv(4096).decode()

            k_buff_size = 4096
            data = b''
            while True:
                part = self.client_socket.recv(k_buff_size)
                data += part
                if len(part) < k_buff_size:
                    break
            data = data.decode()

            if data:
                # JSON 데이터 파싱
                fft_data = json.loads(data)
                frequencies = np.array(fft_data['frequencies'])
                amplitudes = np.array(fft_data['amplitudes'])
                
                # 플롯 업데이트
                self.curve.setData(frequencies, amplitudes)
                
        except BlockingIOError:
            pass  # 새로운 데이터가 없음
        except Exception as e:
            print(f"Error during update: {e}")
            self.connected = False
            self.connect_timer.start()

    def closeEvent(self, event):
        if self.client_socket:
            self.client_socket.close()
        self.server_socket.close()
        event.accept()

def main():
    app = QApplication(sys.argv)
    
    # 스타일 설정 (선택사항)
    app.setStyle('Fusion')
    
    window = FFTVisualizerQt()
    window.show()

    # PyQt5
    # sys.exit(app.exec_())

    # PyQt6
    sys.exit(app.exec())

if __name__ == '__main__':
    main()

'''
CA: 이 PyQt 기반 구현의 주요 특징과 장점을 설명드리겠습니다:

성능 향상:

PyQtGraph는 matplotlib보다 훨씬 빠른 렌더링 속도를 제공합니다
Non-blocking 소켓 사용으로 UI 응답성이 향상됩니다
50ms 간격으로 업데이트하여 부드러운 애니메이션을 제공합니다


향상된 사용자 인터페이스:

그리드 표시
축 레이블과 타이틀
깔끔한 흰색 배경
자동 스케일링


견고한 에러 처리:

연결 끊김 자동 감지
자동 재연결 시도
깔끔한 프로그램 종료


사용하기 쉬운 인터페이스:

창 크기 조절 가능
마우스로 줌/패닝 가능
실시간 데이터 표시



이 구현을 사용하시려면 다음 패키지들을 설치하셔야 합니다:
bashCopypip install PyQt5 pyqtgraph numpy

티마 미니프로그램의 데이터 전송 코드는 이전 버전과 동일하게 사용하시면 됩니다.
이 구현이 어떠신가요? 다음으로 데이터 버퍼링 구현에 대해 설명드릴까요?

HS: 네 부탁합니다.


'''